#include <string.h>
#include <util/delay.h>
#include <avr/io.h>
#include "BTmodule.h"
#include "pin_defs.h"

//Bluetooth module commands

const char cmd_con[] = "CONN=0,11,F6,1,40,D\0";	//Hardcoded address
const char cmd_loss0[] = "LOSSRECONN=0\0";      
const char cmd_loss1[] = "LOSSRECONN=1\0";     
const char cmd_autocon0[] = "STAUTO=0\0";      
const char cmd_autocon1[] = "STAUTO=1\0";      
const char cmd_recon0[] = "STOAUTO=0\0";
const char cmd_recon1[] = "STOAUTO=1\0";
const char cmd_pin[] = "RTPIN=0000\0";    
const char cmd_master[] = "STWMOD=1\0";      
const char cmd_slave[] = "STWMOD=0\0";      
const char cmd_inq1[] = "INQ=1\0";      
const char cmd_inq0[] = "INQ=0\0";      
const char startup_sequence[] = "debug:\r\n";

char BT_frame_buffer[128];
int8_t BT_byte_count;



//
//USART0 commands
//
void USART0_Init( unsigned int baud );
void USART0_Transmit( char data );
void USART0_SendString( char * data );
char USART0_Receive( void );
void USART1_debug( char data );

void BT_init()
{
	int i;
	USART0_Init( USART0_UBBR_VALUE );

	//for (i=0; i<8; i++)
	//  USART1_debug(startup_sequence[i]);

	//Fill with null-termination
	for (i=0; i<128; i++) {
		BT_frame_buffer[i] = '\0';
 	}
	BT_byte_count = 0;
}


//Bluetooth Hello World!
void BT_hello_world( void )
{
  USART0_Transmit('\r');
  USART0_Transmit('\n');
  USART0_Transmit('H');
  USART0_Transmit('e');
  USART0_Transmit('l');
  USART0_Transmit('l');
  USART0_Transmit('o');
  USART0_Transmit(' ');
  USART0_Transmit('W');
  USART0_Transmit('o');
  USART0_Transmit('r');
  USART0_Transmit('l');
  USART0_Transmit('d');
  USART0_Transmit('!');
  USART0_Transmit('\r');
  USART0_Transmit('\n');
}



//Send command
uint8_t BT_send_cmd(const char * cmd)
{
  uint8_t i;
  uint32_t timer = 0UL;
  uint32_t timeout = 50000UL;
  

  USART0_Transmit('\r');
  USART0_Transmit('\n');
  USART0_Transmit('+');

  //Send command
  for (i=0; i<strlen(cmd); i++) {
    USART0_Transmit(cmd[i]);
  }

  USART0_Transmit('\r');
  USART0_Transmit('\n');


  //Error checking
	while (timer < timeout)
	{
		//Check for any pending data
		while (BT_byte_ready()) {
		  BT_read_byte();
		}

		
		//Time out functionality
		_delay_us(10);
		timer++;
	}

	//Check for any pending frame
	while (BT_response_ready()) 
	{
		char response[32];
			
		BT_get_response(response);

		if (strcmp(response+1, cmd) == 0) {
			return 1;
		}
		else if (strcmp(response, "ERROR") == 0) {
			return 0;
		}
     	  
	}

	return 0;
}


uint8_t BT_wait_for_ok( void ) 
{
  uint32_t timer = 0UL;
  uint32_t timeout = 50000UL;

  //Error checking
	while (timer < timeout)
	{
		//Check for any pending data
		while (BT_byte_ready()) {
		  BT_read_byte();
		}
		
		//Time out functionality
		_delay_us(10);
		timer++;
	}


	//Check for any pending frame
	while (BT_response_ready()) 
	{
		char response[32];

		//USART1_debug('t');
			
		BT_get_response(response);

		if (strcmp(response, "OK") == 0) {
			USART1_debug('z');
			return 1;
		}
		else if (strcmp(response, "ERROR") == 0) {
			return 0;
		}
     	  
	}

	return 0;
}


/**
 * Sends the specifiec id
 */
void BT_send_id(char * id, int len)
{
  uint8_t i;

  //Send start character
  USART0_Transmit('+');
  
  //Send id
  for (i=0; i<len; i++) {
    USART0_Transmit(id[i]);
  }

  //Send terminating characters
  USART0_Transmit('\r');
  USART0_Transmit('\n');
  
}

/**
 * Sends a message indicating tag has been removed
 */
void BT_send_removed( void )
{
  //Send start character
  USART0_Transmit('+');
  
  //Send id
  USART0_Transmit('R');
  USART0_Transmit('E');
  USART0_Transmit('M');
  USART0_Transmit('O');
  USART0_Transmit('V');
  USART0_Transmit('E');
  USART0_Transmit('D');

  //Send terminating characters
  USART0_Transmit('\r');
  USART0_Transmit('\n');
  
}



uint8_t BT_connected( void )
{
	return ((PINA & (1<<BT_STATUS)));
}




void BTcmd_set_disconnect()
{
	//Disconnect
	PORTA |= (1<<BT_DISCONNECT);
}

void BTcmd_clear_disconnect()
{
	//Disconnect
	PORTA &= ~(1<<BT_DISCONNECT);
}



//Connect to predefined unit
void BTcmd_connect()
{
  // strcpy(cmd, cmd_con); 					//Prepare command
  BT_send_cmd(cmd_con);                         //Send command
}

//Disable auto-connect if lost connection
void BTcmd_disable_loss_reconnect()
{
  // strcpy(cmd, cmd_loss0); 					//Prepare command
  BT_send_cmd(cmd_loss0);                         //Send command
}

//Disable auto-connect if lost connection
void BTcmd_enable_loss_reconnect()
{
  // strcpy(cmd, cmd_loss1); 					//Prepare command
  BT_send_cmd(cmd_loss1);                         //Send command
}

//Disable auto-connect to last paired unit
void BTcmd_disable_auto_reconnect()
{
  // strcpy(cmd, cmd_autocon0); 					//Prepare command
  BT_send_cmd(cmd_autocon0);                         //Send command
}

//Auto-connect to last paired unit
void BTcmd_set_auto_reconnect()
{
  // strcpy(cmd, cmd_autocon1); 					//Prepare command
  BT_send_cmd(cmd_autocon1);                         //Send command
}

//Permit paired unit to reconnect
void BTcmd_permit_reconnect()
{
  // strcpy(cmd, cmd_recon1); 					//Prepare command
  BT_send_cmd(cmd_recon1);                         //Send command
}

//Do not permit paired unit to reconnect
void BTcmd_forbid_reconnect()
{
  // strcpy(cmd, cmd_recon0); 					//Prepare command
  BT_send_cmd(cmd_recon0);                         //Send command
}

//Set pincode to "0000"
void BTcmd_set_pin_code()
{
  // strcpy(cmd, cmd_pin); 					//Prepare command
  BT_send_cmd(cmd_pin);                         //Send command
}

//Set master mode on
void BTcmd_set_mode_master()
{
  // strcpy(cmd, cmd_master); 					//Prepare command
  BT_send_cmd(cmd_master);                         //Send command
}

//Set slave mode on
void BTcmd_set_mode_slave()
{
  // strcpy(cmd, cmd_slave); 					//Prepare command
  BT_send_cmd(cmd_slave);                         //Send command
}

//Set inquiry on
void BTcmd_set_inquire_on()
{
  //USART1_debug('a');
  // strcpy(cmd, cmd_inq1); 					//Prepare command
  BT_send_cmd(cmd_inq1);                         //Send command
}

//Set inquiry off
void BTcmd_set_inquire_off()
{
  // strcpy(cmd, cmd_inq0); 					//Prepare command
  BT_send_cmd(cmd_inq0);                         //Send command
}






//Check if there is a byte to receive from BT module
uint8_t BT_byte_ready( void )
{
  return (UCSR0A & (1<<RXC0));
}

//Read byte from BT module and store in buffer
void BT_read_byte( void )
{
  //Read the received byte
  unsigned char new_byte = USART0_Receive();
    
  //Save new byte
  BT_frame_buffer[BT_byte_count++] = new_byte;
  //Add null-termination
  BT_frame_buffer[BT_byte_count] = '\0';

}


//Check is a response is ready
uint8_t BT_response_ready( void )
{
  if (BT_byte_count < 3) {
    return 0;
  }

    
  //Make sure last two bytes are \r \n
  if (BT_frame_buffer[BT_byte_count-2] != '\r' || BT_frame_buffer[BT_byte_count-1] != '\n')
  {
    return 0;
  }
  

  //Frame is available
  //for (i=0; i<strlen(BT_frame_buffer); i++)
//	USART1_debug(BT_frame_buffer[i]);
  return 1;
}


//Get ID from frame
void BT_get_response(char * id)
{
  uint8_t i = 0;
  uint8_t u = 0;

  //Ignore preceeding '\r' or '\n' characters
  while (BT_frame_buffer[i] == '\r' || BT_frame_buffer[i] == '\n') {
  	i++;
	
	//Just in case....
	if (i >= 127) {
	  break;
    }
  }
  

  //Get characters
  while (BT_frame_buffer[i] != '\r' && BT_frame_buffer[i] != '\n') {
  	id[u] = BT_frame_buffer[i];
	u++;
	i++;
	
	//Just in case....
	if (i >= 127) {
	  break;
    }
  }

  id[u] = '\0';	//Add null-termination

  //Subtract processed characters from counter value
  BT_byte_count -= i;

  //Shift data to the left
  for (u=0; u<BT_byte_count; u++) {
    BT_frame_buffer[u] = BT_frame_buffer[u+i];	//Shift to the left
	BT_frame_buffer[u+1] = '\0'; 				//Add null-termination to end
  }

}


void BT_process_incoming(void)
{
  uint8_t i;
  uint32_t timer = 0UL;
  uint32_t timeout = 100000UL;

  //Error checking
	while (timer < timeout)
	{
		//Check for any pending data
		while (BT_byte_ready()) {
		  BT_read_byte();
		}
		
		//Time out functionality
		_delay_us(10);
		timer++;
	}


	//Check for any pending frame
	while (BT_response_ready()) 
	{
		char response[32];

		BT_get_response(response);
		for (i=0; i<strlen(response); i++) {
			USART1_debug(response[i]);
		}

		if (strcmp(response, "hello") == 0) {
			BT_hello_world();
		}
		else {
			USART0_SendString("Unknown Command\r\n");
		}
     	  
	}
}


/********************************************
 *
 * USART 0 commands
 *
 ********************************************/

void USART0_Init( unsigned int baud )
{
  /* Set baud rate */
  UBRR0H = (char)(baud>>8);
  UBRR0L = (char)baud;
  
  /* Enable receiver and transmitter */
  UCSR0B = (1<<RXEN0)|(1<<TXEN0);
  
  /* Set frame format: 8data, 1stop bit */
  UCSR0C = (3<<UCSZ00);
}


void USART0_SendString( char * data ) 
{
	uint8_t i;

	for (i=0; i<strlen(data); i++)
	{
		USART0_Transmit(data[i]);
	}
}


void USART0_Transmit( char data )
{
  /* Wait for empty transmit buffer */
  while ( !( UCSR0A & (1<<UDRE0)) )
  ;
  
  /* Put data into buffer, sends the data */
  UDR0 = data;
}



char USART0_Receive( void )
{
  /* Wait for data to be received */
  while ( !(UCSR0A & (1<<RXC0)) )
  ;
  /* Get and return received data from buffer */
  return UDR0;
}


void USART1_debug( char data )
{
  /* Wait for empty transmit buffer */
  while ( !( UCSR1A & (1<<UDRE1)) )
  ;
  
  /* Put data into buffer, sends the data */
  if (data == '\r') {
  	UDR1 = 'c';
	while ( !( UCSR1A & (1<<UDRE1)) );	//Wait until sent
	UDR1 = 'r';
  } else {
	  UDR1 = data;
  }
}

