#define F_CPU 20000000UL 	//20MHz

#include <string.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include "pin_defs.h"
#include "BTmodule.h"
#include "RFIDmodule.h"


const char hello[] = "Hello World!";
char first_read[FRAMESIZE-2];
char second_read[FRAMESIZE-2];
char current_id[FRAMESIZE-2];
char last_id [FRAMESIZE-2];
static uint8_t connected;
static uint8_t rfid_tag_present;


void pin_init()
{
  //Set outputs
   DDRA = (1<<BT_DISCONNECT);
   DDRB = 0xff;
   DDRD = (1<<BT_TX) | (1<<RFID_TX) | (1<<N_ENABLE) | (1<<BT_NRTS);
   
   PORTA |= (1<<BT_DISCONNECT);	//Set disconnect low (active on rising edge)
   PORTD &= ~(1<<BT_NRTS);		//Set RTS low (active low)
   PORTD |= (1<<N_ENABLE);    //Disable RFID reader until connected
   PORTB = 0x00;
}




/*******************
 * Timer functions
 *******************/
void start_timer()
{
  TCCR1B |= (1 << CS12) | (1 << CS10); // Set up timer with CLK/1024
}

void stop_timer()
{
  TCCR1B &= ~((1 << CS12) | (1 << CS11) | (1 << CS10)); // Stop timer
}

void clear_timer()
{
  TCNT1 = 0;
}

int get_timerval_ms()
{
  double timer_resolution_ms = 1000.0 * (1.0 / (F_CPU/1024.0));
  uint32_t elapsed_ms = TCNT1 * timer_resolution_ms;
  
  return elapsed_ms;
}




/*******************
 * Main program
 *******************/
int main(void)
{
  wdt_disable();
  
  //Initialize pins
  pin_init();

  //Give the system some time to start
  _delay_ms(100);

  // Initialise modules
  RFID_init();
  BT_init();
  

/** Receive startup-messages from BT module

 If STAUTO=0 and the module is in slave-mode the
 following message should be transmitted from the module

WORK:SLAVER
+BTSTATE:0
+BTSTATE:1

 The module is now ready to accept commands

 If STAUTO=1, the module will continue with BTSTATE:3 and try to connect.
 In this case, the module is not accepting any commands until connection fails.

 
*/

  //Give bluetooth module some time to get ready
  _delay_ms(4000);
  
  BTcmd_set_inquire_on();
  while (!BT_wait_for_ok())
  {
    BTcmd_set_inquire_on();
  }


  connected = 0;
  rfid_tag_present = 0;
  
  
  while (1)
  {

    //Check bluetooth connection
    if (BT_connected()) 
    {
            
      if (connected == 0) {
        _delay_ms(2000);
        BT_hello_world();
        connected = 1;
      }
		
      RFID_enable();
      start_timer();
		

	    //Check for any pending data
	    while (RFID_byte_ready()) {
	      RFID_read_byte();
      }
  
	    //Check for any pending frame
	    if (RFID_frame_ready()) {
        RFID_disable();
        RFID_get_id(first_read);
        RFID_clear();
        
        // Read tag again within 1 second to make sure it was not noise
        _delay_ms(200);
        clear_timer();
        RFID_enable();
        while (get_timerval_ms() < 800) {
          //Check for any pending data
          while (RFID_byte_ready()) {
            RFID_read_byte();
          }
          //Check for frame ready
          if (RFID_frame_ready()) {
            RFID_disable();
            RFID_get_id(second_read);
            RFID_clear();
            
            //Check if first and second read matches
            if (strncmp(first_read, second_read, FRAMESIZE-2) == 0) {
              //Move current_id to last_id
              strncpy(last_id, current_id, FRAMESIZE-2);
              //Set current_id to second_read
              strncpy(current_id, second_read, FRAMESIZE-2);

              //Check if a different tag from last has been detected
              if (strncmp(last_id, current_id, FRAMESIZE-2) != 0) {
                rfid_tag_present = 1;
                //Send current_id over bluetooth
                BT_send_id(current_id, FRAMESIZE-2);
                clear_timer();
                break;
              }

              //Check if there was no tag present before
              else if (rfid_tag_present == 0) {
                rfid_tag_present = 1;
                //Send current_id over bluetooth
                BT_send_id(current_id, FRAMESIZE-2);
                clear_timer();
                break;
              }
            }
          }
        }
      } 
      else {
        //If 2000 ms has gone by without any tag reads, assume no tag is present.
        if (get_timerval_ms() > 2000) {
          if (rfid_tag_present == 1) {
            BT_send_removed();
          }
          rfid_tag_present = 0;
          clear_timer();
        }
      }
    }
    else {
      connected = 0;
      RFID_disable();
    }
  }
}



