#include <avr/io.h>
#include "RFIDmodule.h"
#include "pin_defs.h"


void USART1_Init( unsigned int baud );
void USART1_Transmit( char data );
char USART1_Receive( void );

static char RFID_frame_buffer[FRAMESIZE];
static uint8_t byte_count;


void RFID_init()
{
	USART1_Init( USART1_UBBR_VALUE );
	byte_count = 0;
}

//Check if there is a byte to receive from RFID reader
int RFID_byte_ready( void )
{
  return (UCSR1A & (1<<RXC1));
}

//Read byte from RFID reader and store in buffer
void RFID_read_byte( void )
{
  //Read the received byte
  char new_byte = USART1_Receive();
    
  //Make sure no overflow happens
  if (byte_count >= FRAMESIZE || byte_count < 0)
    byte_count = 0;
    
  //Save new byte
  RFID_frame_buffer[byte_count++] = new_byte;
  
  //Make sure first byte is the start character
  if (RFID_frame_buffer[0] != STARTCHAR)
    byte_count = 0;
}

int RFID_frame_ready( void )
{
  if (byte_count != FRAMESIZE)
    return 0;
    
  //Make sure last byte is the stop character
  if (RFID_frame_buffer[FRAMESIZE-1] != STOPCHAR)
  {
    byte_count = 0;
    return 0;
  }
  
  //Frame is available
  return 1;
}


//Get ID from frame
void RFID_get_id(char * id)
{
  int i;
  for (i=0; i<FRAMESIZE-2; i++)
    id[i] = RFID_frame_buffer[i+1];
}


void RFID_clear( void )
{
  int i;
  byte_count = 0;
  for (i=0; i<FRAMESIZE; i++)
    RFID_frame_buffer[i] = '\0';
}


void RFID_enable( void )
{
  PORTD &= ~(1<<N_ENABLE);
}

void RFID_disable( void )
{
  PORTD |= (1<<N_ENABLE);
}




/********************************************
 *
 * USART 1 commands
 *
 ********************************************/

void USART1_Init( unsigned int baud )
{
  /* Set baud rate */
  UBRR1H = (char)(baud>>8);
  UBRR1L = (char)baud;
  
  /* Enable receiver and transmitter */
  UCSR1B = (1<<RXEN1)|(1<<TXEN1);
  
  /* Set frame format: 8data, 1stop bit */
  UCSR1C = (3<<UCSZ10);
}



void USART1_Transmit( char data )
{
  /* Wait for empty transmit buffer */
  while ( !( UCSR1A & (1<<UDRE1)) )
  ;
  
  /* Put data into buffer, sends the data */
  UDR1 = data;
}



char USART1_Receive( void )
{
  /* Wait for data to be received */
  while ( !(UCSR1A & (1<<RXC1)) )
  ;
  /* Get and return received data from buffer */
  return UDR1;
}


